@@ -997,6 +997,23 @@ func TestCaseInsensitiveMoveFile(t *testing.T) {
 	r.CheckRemoteItems(t, file2Capitalized)
 }
 
+func TestCaseInsensitiveMoveFileDryRun(t *testing.T) {
+	ctx := context.Background()
+	ctx, ci := fs.AddConfig(ctx)
+	r := fstest.NewRun(t)
+	if !r.Fremote.Features().CaseInsensitive {
+		return
+	}
+
+	file1 := r.WriteObject(ctx, "hello", "world", t1)
+	r.CheckRemoteItems(t, file1)
+
+	ci.DryRun = true
+	err := operations.MoveFile(ctx, r.Fremote, r.Fremote, "HELLO", file1.Path)
+	require.NoError(t, err)
+	r.CheckRemoteItems(t, file1)
+}
+
 func TestMoveFileBackupDir(t *testing.T) {
 	ctx := context.Background()
 	ctx, ci := fs.AddConfig(ctx)
@@ -1240,7 +1257,7 @@ func TestListFormat(t *testing.T) {
 	assert.Equal(t, "a:::b", list.Format(item1))
 
 	list.SetOutput(nil)
-	list.AddModTime()
+	list.AddModTime("")
 	assert.Equal(t, t1.Local().Format("2006-01-02 15:04:05"), list.Format(item0))
 
 	list.SetOutput(nil)
@@ -1272,7 +1289,7 @@ func TestListFormat(t *testing.T) {
 	assert.Equal(t, "1", list.Format(item0))
 
 	list.AddPath()
-	list.AddModTime()
+	list.AddModTime("")
 	list.SetDirSlash(true)
 	list.SetSeparator("__SEP__")
 	assert.Equal(t, "1__SEP__a__SEP__"+t1.Local().Format("2006-01-02 15:04:05"), list.Format(item0))
@@ -1295,7 +1312,7 @@ func TestListFormat(t *testing.T) {
 	list.SetCSV(true)
 	list.AddSize()
 	list.AddPath()
-	list.AddModTime()
+	list.AddModTime("")
 	list.SetDirSlash(true)
 	assert.Equal(t, "1|a|"+t1.Local().Format("2006-01-02 15:04:05"), list.Format(item0))
 	assert.Equal(t, "-1|subdir/|"+t2.Local().Format("2006-01-02 15:04:05"), list.Format(item1))
@@ -1388,6 +1405,33 @@ func TestDirMove(t *testing.T) {
 		fs.GetModifyWindow(ctx, r.Fremote),
 	)
 
+	// Try with a DirMove method that exists but returns fs.ErrorCantDirMove (ex. combine moving across upstreams)
+	// Should fall back to manual move (copy + delete)
+
+	features.DirMove = func(ctx context.Context, src fs.Fs, srcRemote string, dstRemote string) error {
+		return fs.ErrorCantDirMove
+	}
+
+	assert.NoError(t, operations.DirMove(ctx, r.Fremote, "A3", "A4"))
+
+	for i := range files {
+		files[i].Path = strings.ReplaceAll(files[i].Path, "A3/", "A4/")
+	}
+
+	fstest.CheckListingWithPrecision(
+		t,
+		r.Fremote,
+		files,
+		[]string{
+			"A4",
+			"A4/B1",
+			"A4/B2",
+			"A4/B1/C1",
+			"A4/B1/C2",
+			"A4/B1/C3",
+		},
+		fs.GetModifyWindow(ctx, r.Fremote),
+	)
 }
 
 func TestGetFsInfo(t *testing.T) {
@@ -1622,3 +1666,196 @@ func TestTouchDir(t *testing.T) {
 		r.CheckRemoteItems(t, file1, file2, file3)
 	}
 }
+
+var testMetadata = fs.Metadata{
+	// System metadata supported by all backends
+	"mtime": t1.Format(time.RFC3339Nano),
+	// User metadata
+	"potato": "jersey",
+}
+
+func TestMkdirMetadata(t *testing.T) {
+	const name = "dir with metadata"
+	ctx := context.Background()
+	ctx, ci := fs.AddConfig(ctx)
+	ci.Metadata = true
+	r := fstest.NewRun(t)
+	features := r.Fremote.Features()
+	if features.MkdirMetadata == nil {
+		t.Skip("Skipping test as remote does not support MkdirMetadata")
+	}
+
+	newDst, err := operations.MkdirMetadata(ctx, r.Fremote, name, testMetadata)
+	require.NoError(t, err)
+	require.NotNil(t, newDst)
+
+	require.True(t, features.ReadDirMetadata, "Expecting ReadDirMetadata to be supported if MkdirMetadata is supported")
+
+	// Check the returned directory and one read from the listing
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, newDst, testMetadata)
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, fstest.NewDirectory(ctx, t, r.Fremote, name), testMetadata)
+}
+
+func TestMkdirModTime(t *testing.T) {
+	const name = "directory with modtime"
+	ctx := context.Background()
+	r := fstest.NewRun(t)
+	if r.Fremote.Features().DirSetModTime == nil && r.Fremote.Features().MkdirMetadata == nil {
+		t.Skip("Skipping test as remote does not support DirSetModTime or MkdirMetadata")
+	}
+	newDst, err := operations.MkdirModTime(ctx, r.Fremote, name, t2)
+	require.NoError(t, err)
+
+	// Check the returned directory and one read from the listing
+	// newDst may be nil here depending on how the modtime was set
+	if newDst != nil {
+		fstest.CheckDirModTime(ctx, t, r.Fremote, newDst, t2)
+	}
+	fstest.CheckDirModTime(ctx, t, r.Fremote, fstest.NewDirectory(ctx, t, r.Fremote, name), t2)
+}
+
+func TestCopyDirMetadata(t *testing.T) {
+	const nameNonExistent = "non existent directory"
+	const nameExistent = "existing directory"
+	ctx := context.Background()
+	ctx, ci := fs.AddConfig(ctx)
+	ci.Metadata = true
+	r := fstest.NewRun(t)
+	if !r.Fremote.Features().WriteDirMetadata && r.Fremote.Features().MkdirMetadata == nil {
+		t.Skip("Skipping test as remote does not support WriteDirMetadata or MkdirMetadata")
+	}
+
+	// Create a source local directory with metadata
+	newSrc, err := operations.MkdirMetadata(ctx, r.Flocal, "dir with metadata to be copied", testMetadata)
+	require.NoError(t, err)
+	require.NotNil(t, newSrc)
+
+	// First try with the directory not existing
+	newDst, err := operations.CopyDirMetadata(ctx, r.Fremote, nil, nameNonExistent, newSrc)
+	require.NoError(t, err)
+	require.NotNil(t, newDst)
+
+	// Check the returned directory and one read from the listing
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, newDst, testMetadata)
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, fstest.NewDirectory(ctx, t, r.Fremote, nameNonExistent), testMetadata)
+
+	// Then try with the directory existing
+	require.NoError(t, r.Fremote.Rmdir(ctx, nameNonExistent))
+	require.NoError(t, r.Fremote.Mkdir(ctx, nameExistent))
+	existingDir := fstest.NewDirectory(ctx, t, r.Fremote, nameExistent)
+
+	newDst, err = operations.CopyDirMetadata(ctx, r.Fremote, existingDir, "SHOULD BE IGNORED", newSrc)
+	require.NoError(t, err)
+	require.NotNil(t, newDst)
+
+	// Check the returned directory and one read from the listing
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, newDst, testMetadata)
+	fstest.CheckEntryMetadata(ctx, t, r.Fremote, fstest.NewDirectory(ctx, t, r.Fremote, nameExistent), testMetadata)
+}
+
+func TestSetDirModTime(t *testing.T) {
+	const name = "set modtime on existing directory"
+	ctx, ci := fs.AddConfig(context.Background())
+	r := fstest.NewRun(t)
+	if r.Fremote.Features().DirSetModTime == nil && !r.Fremote.Features().WriteDirSetModTime {
+		t.Skip("Skipping test as remote does not support DirSetModTime or WriteDirSetModTime")
+	}
+
+	// Check that we obey --no-update-dir-modtime - this should return nil, nil
+	ci.NoUpdateDirModTime = true
+	newDst, err := operations.SetDirModTime(ctx, r.Fremote, nil, "set modtime on non existent directory", t2)
+	require.NoError(t, err)
+	require.Nil(t, newDst)
+	ci.NoUpdateDirModTime = false
+
+	// First try with the directory not existing - should return an error
+	newDst, err = operations.SetDirModTime(ctx, r.Fremote, nil, "set modtime on non existent directory", t2)
+	require.Error(t, err)
+	require.Nil(t, newDst)
+
+	// Then try with the directory existing
+	require.NoError(t, r.Fremote.Mkdir(ctx, name))
+	existingDir := fstest.NewDirectory(ctx, t, r.Fremote, name)
+
+	newDst, err = operations.SetDirModTime(ctx, r.Fremote, existingDir, "SHOULD BE IGNORED", t2)
+	require.NoError(t, err)
+	require.NotNil(t, newDst)
+
+	// Check the returned directory and one read from the listing
+	// The modtime will only be correct on newDst if it had a SetModTime method
+	if _, ok := newDst.(fs.SetModTimer); ok {
+		fstest.CheckDirModTime(ctx, t, r.Fremote, newDst, t2)
+	}
+	fstest.CheckDirModTime(ctx, t, r.Fremote, fstest.NewDirectory(ctx, t, r.Fremote, name), t2)
+}
+
+func TestDirsEqual(t *testing.T) {
+	ctx := context.Background()
+	ctx, ci := fs.AddConfig(ctx)
+	ci.Metadata = true
+	r := fstest.NewRun(t)
+	if !r.Fremote.Features().WriteDirMetadata && r.Fremote.Features().MkdirMetadata == nil {
+		t.Skip("Skipping test as remote does not support WriteDirMetadata or MkdirMetadata")
+	}
+
+	opt := operations.DirsEqualOpt{
+		ModifyWindow:   fs.GetModifyWindow(ctx, r.Flocal, r.Fremote),
+		SetDirModtime:  true,
+		SetDirMetadata: true,
+	}
+
+	// Create a source local directory with metadata
+	src, err := operations.MkdirMetadata(ctx, r.Flocal, "dir with metadata to be copied", testMetadata)
+	require.NoError(t, err)
+	require.NotNil(t, src)
+
+	// try with nil dst -- should be false
+	equal := operations.DirsEqual(ctx, src, nil, opt)
+	assert.False(t, equal)
+
+	// make a dest with an equal modtime
+	dst, err := operations.MkdirModTime(ctx, r.Fremote, "dst", src.ModTime(ctx))
+	require.NoError(t, err)
+
+	// try with equal modtimes -- should be true
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.True(t, equal)
+
+	// try with unequal modtimes -- should be false
+	dst, err = operations.SetDirModTime(ctx, r.Fremote, dst, "", t2)
+	require.NoError(t, err)
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.False(t, equal)
+
+	// try with unequal modtimes that are within modify window -- should be true
+	halfWindow := opt.ModifyWindow / 2
+	dst, err = operations.SetDirModTime(ctx, r.Fremote, dst, "", src.ModTime(ctx).Add(halfWindow))
+	require.NoError(t, err)
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.True(t, equal)
+
+	// test ignoretimes -- should be false
+	ci.IgnoreTimes = true
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.False(t, equal)
+
+	// test immutable -- should be true
+	ci.IgnoreTimes = false
+	ci.Immutable = true
+	dst, err = operations.SetDirModTime(ctx, r.Fremote, dst, "", t3)
+	require.NoError(t, err)
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.True(t, equal)
+
+	// test dst newer than src with --update -- should be true
+	ci.Immutable = false
+	ci.UpdateOlder = true
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.True(t, equal)
+
+	// test no SetDirModtime or SetDirMetadata -- should be true
+	ci.UpdateOlder = false
+	opt.SetDirMetadata, opt.SetDirModtime = false, false
+	equal = operations.DirsEqual(ctx, src, dst, opt)
+	assert.True(t, equal)
+}
